@@ -15,11 +15,19 @@ import datetime
 import tempfile
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-from gallery_dl import formatter, text, util  # noqa E402
+from gallery_dl import formatter, text, util, config  # noqa E402
+
+try:
+    import jinja2
+except ImportError:
+    jinja2 = None
 
 
 class TestFormatter(unittest.TestCase):
 
+    def tearDown(self):
+        config.clear()
+
     kwdict = {
         "a": "hElLo wOrLd",
         "b": "äöü",
@@ -34,6 +42,7 @@ class TestFormatter(unittest.TestCase):
         ],
         "n": None,
         "s": " \n\r\tSPACE    ",
+        "S": " \n\r\tS  P         A\tC\nE    ",
         "h": "<p>foo </p> &amp; bar <p> </p>",
         "u": "&#x27;&lt; / &gt;&#x27;",
         "t": 1262304000,
@@ -56,6 +65,7 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{a!c}", "Hello world")
         self._run_test("{a!C}", "Hello World")
         self._run_test("{s!t}", "SPACE")
+        self._run_test("{S!t}", "S  P         A\tC\nE")
         self._run_test("{a!U}", self.kwdict["a"])
         self._run_test("{u!U}", "'< / >'")
         self._run_test("{a!H}", self.kwdict["a"])
@@ -63,8 +73,8 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{u!H}", "'< / >'")
         self._run_test("{n!H}", "")
         self._run_test("{a!s}", self.kwdict["a"])
-        self._run_test("{a!r}", "'" + self.kwdict["a"] + "'")
-        self._run_test("{a!a}", "'" + self.kwdict["a"] + "'")
+        self._run_test("{a!r}", f"'{self.kwdict['a']}'")
+        self._run_test("{a!a}", f"'{self.kwdict['a']}'")
         self._run_test("{b!a}", "'\\xe4\\xf6\\xfc'")
         self._run_test("{a!S}", self.kwdict["a"])
         self._run_test("{l!S}", "a, b, c")
@@ -85,6 +95,8 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{a!n}", 11)
         self._run_test("{l!n}", 3)
         self._run_test("{d!n}", 3)
+        self._run_test("{s!W}", "SPACE")
+        self._run_test("{S!W}", "S P A C E")
         self._run_test("{i_str!i}", 12345)
         self._run_test("{i_str!f}", 12345.0)
         self._run_test("{f_str!f}", 12.45)
@@ -127,7 +139,7 @@ class TestFormatter(unittest.TestCase):
         self._run_test("{missing}"     , replacement, default)
         self._run_test("{missing.attr}", replacement, default)
         self._run_test("{missing[key]}", replacement, default)
-        self._run_test("{missing:?a//}", "a" + default, default)
+        self._run_test("{missing:?a//}", f"a{default}", default)
 
     def test_fmt_func(self):
         self._run_test("{t}" , self.kwdict["t"] , None, int)
@@ -432,11 +444,11 @@ class TestFormatter(unittest.TestCase):
 
             with open(path1, "w") as fp:
                 fp.write("{a}")
-            fmt1 = formatter.parse("\fT " + path1)
+            fmt1 = formatter.parse(f"\fT {path1}")
 
             with open(path2, "w") as fp:
                 fp.write("{a!u:Rh/C/}\nFooBar")
-            fmt2 = formatter.parse("\fT " + path2)
+            fmt2 = formatter.parse(f"\fT {path2}")
 
         self.assertEqual(fmt1.format_map(self.kwdict), self.kwdict["a"])
         self.assertEqual(fmt2.format_map(self.kwdict), "HELLO WORLD\nFooBar")
@@ -446,15 +458,18 @@ class TestFormatter(unittest.TestCase):
 
     def test_expression(self):
         self._run_test("\fE a", self.kwdict["a"])
-        self._run_test("\fE name * 2 + ' ' + a", "{}{} {}".format(
-            self.kwdict["name"], self.kwdict["name"], self.kwdict["a"]))
+        self._run_test(
+            "\fE name * 2 + ' ' + a",
+            f"{self.kwdict['name']}{self.kwdict['name']} {self.kwdict['a']}")
 
     def test_fstring(self):
         self._run_test("\fF {a}", self.kwdict["a"])
-        self._run_test("\fF {name}{name} {a}", "{}{} {}".format(
-            self.kwdict["name"], self.kwdict["name"], self.kwdict["a"]))
-        self._run_test("\fF foo-'\"{a.upper()}\"'-bar",
-                       """foo-'"{}"'-bar""".format(self.kwdict["a"].upper()))
+        self._run_test(
+            "\fF {name}{name} {a}",
+            f"{self.kwdict['name']}{self.kwdict['name']} {self.kwdict['a']}")
+        self._run_test(
+            "\fF foo-'\"{a.upper()}\"'-bar",
+            f"""foo-'"{self.kwdict['a'].upper()}"'-bar""")
 
     def test_template_fstring(self):
         with tempfile.TemporaryDirectory() as tmpdirname:
@@ -463,19 +478,105 @@ class TestFormatter(unittest.TestCase):
 
             with open(path1, "w") as fp:
                 fp.write("{a}")
-            fmt1 = formatter.parse("\fTF " + path1)
+            fmt1 = formatter.parse(f"\fTF {path1}")
 
             with open(path2, "w") as fp:
                 fp.write("foo-'\"{a.upper()}\"'-bar")
-            fmt2 = formatter.parse("\fTF " + path2)
+            fmt2 = formatter.parse(f"\fTF {path2}")
 
         self.assertEqual(fmt1.format_map(self.kwdict), self.kwdict["a"])
         self.assertEqual(fmt2.format_map(self.kwdict),
-                         """foo-'"{}"'-bar""".format(self.kwdict["a"].upper()))
+                         f"""foo-'"{self.kwdict['a'].upper()}"'-bar""")
 
         with self.assertRaises(OSError):
             formatter.parse("\fTF /")
 
+    @unittest.skipIf(jinja2 is None, "no jinja2")
+    def test_jinja(self):
+        formatter.JinjaFormatter.env = None
+
+        self._run_test("\fJ {{a}}", self.kwdict["a"])
+        self._run_test(
+            "\fJ {{name}}{{name}} {{a}}",
+            f"{self.kwdict['name']}{self.kwdict['name']} {self.kwdict['a']}")
+        self._run_test(
+            "\fJ foo-'\"{{a | upper}}\"'-bar",
+            f"""foo-'"{self.kwdict['a'].upper()}"'-bar""")
+
+    @unittest.skipIf(jinja2 is None, "no jinja2")
+    def test_template_jinja(self):
+        formatter.JinjaFormatter.env = None
+
+        with tempfile.TemporaryDirectory() as tmpdirname:
+            path1 = os.path.join(tmpdirname, "tpl1")
+            path2 = os.path.join(tmpdirname, "tpl2")
+
+            with open(path1, "w") as fp:
+                fp.write("{{a}}")
+            fmt1 = formatter.parse(f"\fTJ {path1}")
+
+            with open(path2, "w") as fp:
+                fp.write("foo-'\"{{a | upper}}\"'-bar")
+            fmt2 = formatter.parse(f"\fTJ {path2}")
+
+        self.assertEqual(fmt1.format_map(self.kwdict), self.kwdict["a"])
+        self.assertEqual(fmt2.format_map(self.kwdict),
+                         f"""foo-'"{self.kwdict['a'].upper()}"'-bar""")
+
+        with self.assertRaises(OSError):
+            formatter.parse("\fTJ /")
+
+    @unittest.skipIf(jinja2 is None, "no jinja2")
+    def test_template_jinja_opts(self):
+        formatter.JinjaFormatter.env = None
+
+        with tempfile.TemporaryDirectory() as tmpdirname:
+            path_filters = os.path.join(tmpdirname, "jinja_filters.py")
+            path_template = os.path.join(tmpdirname, "jinja_template.txt")
+
+            config.set((), "jinja", {
+                "environment": {
+                    "variable_start_string": "(((",
+                    "variable_end_string"  : ")))",
+                    "keep_trailing_newline": True,
+                },
+                "filters": path_filters,
+            })
+
+            with open(path_filters, "w") as fp:
+                fp.write(r"""
+import re
+
+def datetime_format(value, format="%H:%M %d-%m-%y"):
+    return value.strftime(format)
+
+def sanitize(value):
+    return re.sub(r"\s+", " ", value.strip())
+
+__filters__ = {
+    "dt_fmt": datetime_format,
+    "sanitize_whitespace": sanitize,
+}
+""")
+
+            with open(path_template, "w") as fp:
+                fp.write("""\
+Present Day  is ((( dt | dt_fmt("%B %d, %Y") )))
+Present Time is ((( dt | dt_fmt("%H:%M:%S") )))
+
+Hello ((( s | sanitize_whitespace ))).
+I hope there is enough "(((S|sanitize_whitespace)))" for you.
+""")
+            fmt = formatter.parse(f"\fTJ {path_template}")
+
+        self.assertEqual(fmt.format_map(self.kwdict), """\
+Present Day  is January 01, 2010
+Present Time is 00:00:00
+
+Hello SPACE.
+I hope there is enough "S P A C E" for you.
+""")
+
     def test_module(self):
         with tempfile.TemporaryDirectory() as tmpdirname:
             path = os.path.join(tmpdirname, "testmod.py")
@@ -511,14 +612,14 @@ def noarg():
             finally:
                 sys.path.pop(0)
 
-            fmt3 = formatter.parse("\fM " + path + ":gentext")
-            fmt4 = formatter.parse("\fM " + path + ":lengths")
+            fmt3 = formatter.parse(f"\fM {path}:gentext")
+            fmt4 = formatter.parse(f"\fM {path}:lengths")
 
         self.assertEqual(fmt1.format_map(self.kwdict), "'Title' by Name")
-        self.assertEqual(fmt2.format_map(self.kwdict), "142")
+        self.assertEqual(fmt2.format_map(self.kwdict), "168")
 
         self.assertEqual(fmt3.format_map(self.kwdict), "'Title' by Name")
-        self.assertEqual(fmt4.format_map(self.kwdict), "142")
+        self.assertEqual(fmt4.format_map(self.kwdict), "168")
 
         with self.assertRaises(TypeError):
             self.assertEqual(fmt0.format_map(self.kwdict), "")
